/**************************************************************************************
**
**  The Osiris application compares one catalog of executable files
**  with another catalog of executable files. Osiris catalogs specified
**  directories of files (including MD5 hashes, modification dates,
**  and file attributes into a specified database.
**
**  Copyright (C) 2000  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
**************************************************************************************/

/**************************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      database_handler.c
**    Author:    Brian Wotring
**
**    Date:      January 27, 2000.
**    Project:   osiris ( UNIX ).
**
**************************************************************************************/

#include "database_handler.h"


/**************************************************************************************
**
**    Function: open_database_for_read
**
**    Purpose:  attempts to open the database.  Currently, this is gdbm.  the
**		database can is opened read only.
**
**    Return:   Upon success, a pointer to the newly allocated database
**   		structure, NULL otherwise.
**
**
**************************************************************************************/

database * open_database_for_read( const char *file_path )
{
    database *db;

    db = (database *)safemalloc( sizeof( database ) );
    db->database_file = gdbm_open( (char *)file_path, 0, GDBM_READER, 0, NULL );
 
    return db;
}


/**************************************************************************************
**
**    Function: create_database_fort_write
**
**    Purpose:  attempts to open the database.  Currently, this is gdbm.  the
**		database can is created, if it exists, it is overwritten.  The
**		database is read/write.
**
**    Return:   Upon success, a pointer to the newly allocated database
**   		structure, NULL otherwise.
**
**
**************************************************************************************/

database * create_database_for_write( const char *file_path )
{
    database *db;

    db = (database *)safemalloc( sizeof( database ) );
    db->database_file = gdbm_open( (char *)file_path, 0, GDBM_WRCREAT, 00700, NULL );

    return db;
}

/**************************************************************************************
**
**    Function: get_first_key
**
**    Purpose:  for sequential access to the database, fetch the first key.  There
**		is no order to the access this way so be careful.
**
**    Return:   Upon success, a pointer to the fetched key is returned
**		otherwise, return NULL.
**
**************************************************************************************/

char * get_first_key( database *db )
{
    datum key;
    char *result;

    if( db == NULL )
	return NULL;

    key    = gdbm_firstkey( db->database_file );
    result = (char *)safemalloc( key.dsize + 1 );

    memcpy( result, key.dptr, key.dsize );
    result[key.dsize] = '\0';

    free( key.dptr );
    return result;
}

/**************************************************************************************
**
**    Function: get_next_key
**
**    Purpose:  used with get_first_key to provide a means of sequential access
**		to the database, in no specific order.
**
**    Return:   Upon success, a pointer to the fetched key is returned
**		otherwise, return NULL.
**
**************************************************************************************/

char * get_next_key( database *db, const char *key )
{
    datum fetch_key;
    char *result;

    if( key == NULL || db == NULL )
	return NULL;

    fetch_key.dptr  = (char *)key;
    fetch_key.dsize = strlen( key );

    fetch_key = gdbm_nextkey( db->database_file, fetch_key );

    if( fetch_key.dptr == NULL )
	return NULL;

    result    = (char *)safemalloc( fetch_key.dsize + 1 );
    memcpy( result, fetch_key.dptr, fetch_key.dsize );
    result[fetch_key.dsize] = '\0';

    free( fetch_key.dptr );
    return result;
}

/**************************************************************************************
**
**    Function: key_exists
**
**    Purpose:  determine whether or not an entry exists in the database with
**		the specified key.
**
**    Return:   returns 1 if key exists in database, 0 otherwise.
**
**
**************************************************************************************/

int key_exists( database *db, const char *key )
{
    datum fetch_key;
 
    if( key == NULL || db == NULL )
	return 0;

    fetch_key.dptr  = (char *)key;
    fetch_key.dsize = strlen( key );

    if( gdbm_exists( db->database_file, fetch_key ) )
	return 1;

    return 0;
}

/**************************************************************************************
**
**    Function: get_data_for_key
**
**    Purpose:  attempts to find the key in the database.  If it exists, the
**		data associated with the key is returned.
**
**    Return:   upon success, returns the data associated with the key in the 
**		database, otherwise, returns NULL.
**
**************************************************************************************/

char * get_data_for_key( database *db, const char *key )
{
    datum data;
    datum fetch_key;

    char *result;

    if( key == NULL || db == NULL )
	return NULL;

    fetch_key.dptr  = (char *)key;
    fetch_key.dsize = strlen( key );

    data = gdbm_fetch( db->database_file, fetch_key );
    result = (char *)safemalloc( data.dsize + 1 );

    memcpy( result, data.dptr, data.dsize );
    result[data.dsize] = '\0';

    free( data.dptr );
    return result;
}

/**************************************************************************************
**
**    Function: set_data_for_key
**
**    Purpose:  stores the data in the database with the specified key.  If the
**		key already exists, it is replaced.
**
**    Return:   upon success, 1 is returned, otherwise returns 0.
**
**
**************************************************************************************/

int set_data_for_key( database *db, const char *data, const char *key )
{
    datum store_key;
    datum store_data;

    if( db == NULL || data == NULL || key == NULL )
	return 0;

    store_key.dptr   = (char *)key;
    store_key.dsize  = strlen( key );

    store_data.dptr  = (char *)data;
    store_data.dsize = strlen( data );

    if( gdbm_store( db->database_file, store_key, store_data, GDBM_REPLACE ) != 0 )
	return -1;

    return 1;
}

/**************************************************************************************
**
**    Function: at_end
**
**    Purpose:  queries the database structure to determine if the at_end flag
**		was set.  This is done when the last get_next_key returns NULL.
**
**    Return:   returns 1 if database is at end of sequential access, 0 otherwise.
**
**
**************************************************************************************/

int at_end( database *db )
{
    if( db == NULL )
	return 1;

    return db->end;
}

/**************************************************************************************
**
**    Function: close_database
**
**    Purpose:  closes the database if it is opened, otherwise, does nothing.
**
**    Return:   none.
**
**
**************************************************************************************/

void close_database( database *db )
{
    if( db == NULL )
	return;

    if( db->open )
	gdbm_close( db->database_file );
}

