
/**************************************************************************************
**
**  The Osiris application compares one catalog of executable files
**  with another catalog of executable files. Osiris catalogs specified
**  directories of files (including MD5 hashes, modification dates,
**  and file attributes into a specified database.
**
**  Copyright (C) 2000  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
**************************************************************************************/

/**************************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      entry.c
**    Author:    Brian Wotring
**
**    Date:      January 27, 2000.
**    Project:   osiris ( UNIX ).
**
**************************************************************************************/

#include "entry.h"

/**************************************************************************************
**
**    Function: get_entry_from_file
**
**    Purpose:  creates an osiris entry for a file according to the format
**		specified below.
**
**    Return:   Upon success, a pointer to the newly allocated entry for
**		the file, NULL otherwise.
**
** 		entries are composed of a string of file specifics in the 
** 		following format:                                       
**                                                         
**  		hash;mod-date;attributes;uid;gid;suid;                  
**
**************************************************************************************/

entry * get_entry_from_file( const char *file_path )
{
    char *hash;
    char *date;
    char *attributes;

    char *uid;
    char *gid;

    struct stat file_stats;
    entry *file_entry;

    int status = 1;

    if( stat( file_path, &file_stats ) < 0 )
	return NULL;

    if( ( file_entry = init_entry() ) == NULL )
	return NULL;

    file_entry->key  = (char *)safemalloc( ( sizeof( char ) * strlen( file_path ) ) + 1 );
    strcpy( file_entry->key, file_path );
    file_entry->key[strlen(file_path)] = '\0';

    file_entry->data = (char *)safemalloc( sizeof( char ) * 200 );
    file_entry->data[0] = '\0';

    /* get hash of file. */

    if( ( hash = hash_file_md5( file_path ) ) != NULL )
    {
        strcat( file_entry->data, hash );
        strcat( file_entry->data, ";" );
    }

    /* get mod date of file. */

    if( ( date = get_file_date( file_path ) ) != NULL )
    {
        strcat( file_entry->data, date );
        strcat( file_entry->data, ";" );
    }

    /* get attributes of file. */

    if( ( attributes = get_file_attributes( file_path ) ) != NULL )
    {
        strcat( file_entry->data, attributes );
        strcat( file_entry->data, ";" );
    }

    /* get uid. */

    if( ( uid = get_file_uid( file_path ) ) != NULL )
    {
        strcat( file_entry->data, uid );
        strcat( file_entry->data, ";" );
    }

    /* get gid. */
 
    if( ( gid = get_file_gid( file_path ) ) != NULL )
    {
        strcat( file_entry->data, gid );
        strcat( file_entry->data, ";" );
    }

    /* get suid status. */

    strcat( file_entry->data, ( ( file_is_suid( file_path ) > 0 ) ? "1" : "0" ) );

    if( hash != NULL )       free( hash );       else status = 0;
    if( date != NULL )       free( date );       else status = 0;
    if( attributes != NULL ) free( attributes ); else status = 0;
    if( uid != NULL )        free( uid );        else status = 0;
    if( gid != NULL )        free( gid );        else status = 0;

    if( status != 0 )
        return file_entry;

    destroy_entry( file_entry );
    return NULL;
}

/**************************************************************************************
**
**    Function: get_file_date
**
**    Purpose:  attempt to get the modification date of the file specified
**		as a string.
**
**    Return:   Upon success, a pointer to the newly allocated date string for
**		the file, NULL otherwise.
**
**************************************************************************************/

char * get_file_date( const char *file_path )
{
    char buffer[50];
    char *date;

    struct stat file_stats;
   
    if( stat( file_path, &file_stats ) < 0 )
	return NULL;

    strcpy( buffer, ctime( &file_stats.st_mtime ) );
    buffer[strlen(buffer)-1] = '\0';                   /* remove newline from time. */

    date = (char *)safemalloc( strlen( buffer ) );
    strcpy( date, buffer );
    
    return date;   
}

/**************************************************************************************
**
**    Function: get_file_attributes
**
**    Purpose:  attempt to get the access attributes of the file specified
**		as a string.
**
**    Return:   Upon success, a pointer to the newly allocated attribute string for
**		the file, NULL otherwise.
**
**************************************************************************************/

char * get_file_attributes( const char *file_path )
{
    struct stat file_stats;
    char *attributes;

    if( stat( file_path, &file_stats ) < 0 )
        return NULL;

    attributes = (char *)safemalloc( 11 );

    attributes[0] =  S_ISDIR( file_stats.st_mode ) ? 'd' : 
                     S_ISLNK( file_stats.st_mode  ) ? 'l' :
                     S_ISCHR( file_stats.st_mode  ) ? 'c' :
                     S_ISBLK( file_stats.st_mode  ) ? 'b' :
                     S_ISFIFO( file_stats.st_mode ) ? 'f' :
                     S_ISSOCK( file_stats.st_mode ) ? 's' :
                     S_ISREG( file_stats.st_mode  ) ? '-' : '?';

    
    attributes[1] = file_stats.st_mode & S_IRUSR ? 'r' : '-';
    attributes[1] = file_stats.st_mode & S_IRUSR ? 'r' : '-';
    attributes[2] = file_stats.st_mode & S_IWUSR ? 'w' : '-';
    attributes[3] = file_stats.st_mode & S_IXUSR ? file_stats.st_mode & S_ISUID ? 's' : 'x' : '-';
    attributes[4] = file_stats.st_mode & S_IRGRP ? 'r' : '-';
    attributes[5] = file_stats.st_mode & S_IWGRP ? 'w' : '-';
    attributes[6] = file_stats.st_mode & S_IXGRP ? file_stats.st_mode & S_ISGID ? 's' : 'x' : '-';
    attributes[7] = file_stats.st_mode & S_IROTH ? 'r' : '-';
    attributes[8] = file_stats.st_mode & S_IWOTH ? 'w' : '-';
    attributes[9] = file_stats.st_mode & S_IXOTH ? file_stats.st_mode & S_ISVTX ? 'T' : 'x' : file_stats.st_mode & S_ISVTX ? 'T' : '-';
    attributes[10] = '\0';

    return attributes;
}


/**************************************************************************************
**
**    Function: get_file_uid
**
**    Purpose:  attempt to get the uid  of the file specified
**		as a string.
**
**    Return:   Upon success, a pointer to the newly allocated uid string for
**		the file, NULL otherwise.
**
**************************************************************************************/

char * get_file_uid( const char *file_path )
{
    char buffer[20];
    char *uid;

    struct stat file_stats;

    if( stat( file_path, &file_stats ) < 0 )
	return NULL;

    sprintf( buffer, "%d", (int)file_stats.st_uid );
    uid = (char *)safemalloc( strlen( buffer ) );

    strcpy( uid, buffer );
    return uid;
}

/**************************************************************************************
**
**    Function: get_file_gid
**
**    Purpose:  attempt to get the gid  of the file specified
**		as a string.
**
**    Return:   Upon success, a pointer to the newly allocated gid string for
**		the file, NULL otherwise.
**
**************************************************************************************/

char * get_file_gid( const char *file_path )
{
    char buffer[20];
    char *gid;

    struct stat file_stats;

    if( stat( file_path, &file_stats ) < 0 )
	return NULL;

    sprintf( buffer, "%d", (int)file_stats.st_gid );
    gid = (char *)safemalloc( strlen( buffer ) );

    strcpy( gid, buffer );
    return gid;
}


/**************************************************************************************
**
**    Function: destroy_entry
**
**    Purpose:  frees the memory associated with the file entry specified.
**
**    Return:   none.
**
**************************************************************************************/

void destroy_entry( entry *file_entry )
{
    if( file_entry == NULL )
    {
	log_message( "file entry is null." );
	return;
    }
 
    if( file_entry->key )
        free( file_entry->key );

    if( file_entry->data )
        free( file_entry->data );

    free( file_entry );
}


/**************************************************************************************
**
**    Function: init_entry
**
**    Purpose:  allocate for an entry object, initialize members to NULL.
**
**    Return:   Upon success, a pointer to the newly allocated entry
**		object, NULL otherwise.
**
**************************************************************************************/

entry * init_entry()
{
    entry *e = (entry *)safemalloc( sizeof( entry ) );

    e->key  = NULL;
    e->data = NULL;

    return e;
}

