
/**************************************************************************************
**
**  The Osiris application compares one catalog of executable files
**  with another catalog of executable files. Osiris catalogs specified
**  directories of files (including MD5 hashes, modification dates,
**  and file attributes into a specified database.
**
**  Copyright (C) 2000  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
**************************************************************************************/

/**************************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      file_filter.c
**    Author:    Brian Wotring
**
**    Date:      January 27, 2000.
**    Project:   osiris ( UNIX ).
**
**************************************************************************************/

#include "file_filter.h"


/**************************************************************************************
**
**    Function: file_is_elf
**
**    Purpose:  determine whether or not the file is in elf file format by
**		reading the first 4 bytes of the file.
**
**    Return:   returns 1 i file is elf format, -1 on error, 0 if not elf.
**
**************************************************************************************/

int file_is_elf( const char *file_path )
{
    FILE *file;
    struct stat file_stats;

    char buffer[5];

    if( ( file = fopen( file_path, "r" ) ) == NULL )
	return -1;

    if( stat( file_path, &file_stats ) < 0 )
    {
	fclose( file );
	return -1;
    }

    if( ( file_stats.st_size < 4 ) || ( fread( buffer, 1, 4, file ) < 4 ) )
    {
	fclose( file );
        return 0;
    }

    buffer[4] = '\0';

    /* check for shell scripts, or ELF headers. */

    fclose( file );
    return ( strstr( buffer, "ELF" ) ) ? 1 : 0;
}

/**************************************************************************************
**
**    Function: file_is_shell_script
**
**    Purpose:  determine whether or not the file is shell script by
**		reading the first 9 bytes of the file.
**
**    Return:   returns 1 i file is a script, -1 on error, 0 if not elf.
**
**************************************************************************************/

int file_is_shell_script( const char *file_path )
{
    FILE *file;
    struct stat file_stats;

    char buffer[10];

    if( ( file = fopen( file_path, "r" ) ) == NULL )
	return -1;

    if( stat( file_path, &file_stats ) < 0 )
    {
	fclose( file );
	return -1;
    }

    if( ( file_stats.st_size < 9 ) || ( fread( buffer, 1, 9, file ) < 9 ) )
    {
	fclose( file );
        return 0;
    }

    buffer[9] = '\0';

    /* check for shell scripts, or ELF headers. */

    fclose( file );
    return ( strstr( buffer, "#!/bin/sh" ) ) ? 1 : 0;
}


/**************************************************************************************
**
**    Function: file_is_executable
**
**    Purpose:  determine whether or not the file is an executable by
**		reading execute bit of the file.
**
**    Return:   returns 1 i file is executable, -1 on error, 0 if not elf.
**
**************************************************************************************/

int file_is_executable( const char *file_path )
{
    struct stat file_stats;

    if( stat( file_path, &file_stats ) < 0 )
        return -1;

    if( ( file_stats.st_mode & S_IXUSR ) || ( file_stats.st_mode & S_ISUID ) )
        return 1;

    return 0;
}

/**************************************************************************************
**
**    Function: file_is_allowed
**
**    Purpose:  determine whether or not the file passes through the filter
**		specified.  This is done by applying the mask if necessary
**		
**
**    Return:   returns greater than zero if passes, 0 if does not pass.
**
**************************************************************************************/

int file_is_allowed( file_filter *filter, const char *file_path )
{
    unsigned int mask;
    unsigned int is_allowed = 0;

    if( filter == NULL || file_is_empty( file_path ) )
	return 0;

    mask = filter->definition;

    /* let everything pass through the filter if FILTER_ALL is set. */

    if( mask == FILTER_ALL )
        return 1;

    if( ( FILTER_ELF & mask ) && file_is_elf( file_path ) )
	is_allowed++;

    if( ( FILTER_EXECUTABLE & mask ) && file_is_executable( file_path ) )
	is_allowed++;

    if( ( FILTER_SCRIPT & mask ) && file_is_shell_script( file_path ) )
	is_allowed++;

    return is_allowed;
}


/**************************************************************************************
**
**    Function: set_file_filter
**
**    Purpose:  sets the filter definition, overwriting any existing definition.
**
**    Return:   none.
**
**************************************************************************************/

void set_file_filter( file_filter *filter, int filter_definition )
{
    if( filter == NULL )
	return;

    filter->definition = filter_definition;
}


/**************************************************************************************
**
**    Function: add_to_file_filter
**
**    Purpose:  add a file type to be allowed to the filter definition.
**
**    Return:   none.
**
**************************************************************************************/

void add_to_file_filter( file_filter *filter, int filter_definition )
{
    if( filter == NULL )
	return;

    filter->definition = ( filter->definition | filter_definition );
}

