
/**************************************************************************************
**
**  The Osiris application compares one catalog of executable files
**  with another catalog of executable files. Osiris catalogs specified
**  directories of files (including MD5 hashes, modification dates,
**  and file attributes into a specified database.
**
**  Copyright (C) 2000  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
**************************************************************************************/

/**************************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      file_utilities.c
**    Author:    Brian Wotring
**
**    Date:      January 27, 2000.
**    Project:   osiris ( UNIX ).
**
**************************************************************************************/

#include "file_utilities.h"


/**************************************************************************************
**
**    Function: file_is_suid
**
**    Purpose:  determines whether or not the file specified has the suid bit set.
**
**    Return:   returns 1 if bit is set, 0 if not, -1 on error.
**
**************************************************************************************/

int file_is_suid( const char *file_path )
{
    struct stat file_stats;

    if( stat( file_path, &file_stats ) < 0 )
	return -1;

    if( file_stats.st_mode & S_ISUID )
	return 1;

    return 0;
}


/**************************************************************************************
**
**    Function: file_is_empty
**
**    Purpose:  determines whether or not the file is a zero length file.
**
**    Return:   returns 1 if file is zero lengh, 0 otherwise.
**
**************************************************************************************/

int file_is_empty( const char *file_path )
{
    struct stat file_stats;

    if( stat( file_path, &file_stats ) < 0 )
	return 0;

    if( file_stats.st_size == 0 )
	return 1;

    return 0;
}


/**************************************************************************************
**
**    Function: file_is_directory
**
**    Purpose:  determine whether or not the file specified is a directory.
**
**    Return:   returns 1 if directory, 0 if not directory, -1 on error.
**
**
**************************************************************************************/

int file_is_directory( const char *file_path )
{
    struct stat file_stats;

    if( stat( file_path, &file_stats ) < 0 )
	return -1;

    if( S_ISDIR( file_stats.st_mode ) )
	return 1;

    return 0;
}


/**************************************************************************************
**
**    Function: file_is_symlink
**
**    Purpose:  determines whether or not the file specified is a symbolic
**		link of sorts.
**
**    Return:   returns 1 i file is link, 0 if not link, -1 on error.
**
**************************************************************************************/

int file_is_symlink( const char *file_path )
{
    struct stat file_stats;

    if( lstat( file_path, &file_stats ) < 0 )
	return -1;

    if( S_ISLNK( file_stats.st_mode ) )
	return 1;

    return 0;
}


/**************************************************************************************
**
**    Function: file_is_symlink_to_directory
**
**    Purpose:  similar to printf, but will add a new line, and flush
**		the output stream.
**
**    Return:   returns positive if symlink to directory, 0 if not, -1 on error.
**
**
**************************************************************************************/

int file_is_symlink_to_directory( const char *file_path )
{
    int directory;
    int symlink;

    directory = file_is_directory( file_path );
    symlink   = file_is_symlink( file_path );
 
    if( directory < 0 || symlink < 0 )
 	return -1;

    return ( directory & symlink );
}

