
/**************************************************************************************
**
**  The Osiris application compares one catalog of executable files
**  with another catalog of executable files. Osiris catalogs specified
**  directories of files (including MD5 hashes, modification dates,
**  and file attributes into a specified database.
**
**  Copyright (C) 2000  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
** 
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
**************************************************************************************/

/**************************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      osiris.c
**    Author:    Brian Wotring
**
**    Date:      January 27, 2000.
**    Project:   osiris ( UNIX ).
**
**************************************************************************************/


#include "osiris.h"


/**************************************************************************************
**
**    Function: int main
**
**    Purpose:  system driver for osiris
**
**    Return:   zero, if all goes well.
**
**************************************************************************************/

int main( int argument_count, char *argument_list[] )
{
    init( argument_count, argument_list );

    process();
    quit();

    return 0;
}


/**************************************************************************************
**
**    Function: init 
**
**    Purpose:  check the arguments and try to open the database files.
**
**    Return:   none.
**
**************************************************************************************/

void init( int argument_count, char *argument_list[]  )
{
    time_t current_time = time( NULL );
    strcpy( start_time, ctime( &current_time ) );
    start_time[strlen( start_time ) - 1] = '\0';

    set_file_filter( &filter, ( FILTER_ELF | FILTER_EXECUTABLE | FILTER_SCRIPT ) );

    check_arguments( argument_count, argument_list );
    open_database_files();

    if( !quiet_mode )
        print_startup_info();
}

/**************************************************************************************
**
**    Function: int process 
**
**    Purpose:  go through all of the directories specified by the user
**		in the directory_list and process them.
**
**    Return:   none.
**
**************************************************************************************/

void process()
{
    int index;

    /* main loop of application, goes through the directory list and */
    /* recursively processes the directories.                        */

    for( index = 0; index < directory_list_size; index++ )
        process_directory( directory_list[index] );
}


/**************************************************************************************
**
**    Function: process_directory
**
**    Purpose:  recursive function that collects all of the files for a
**  		directory and sends them to the process_file function.
**
**    Return:   none.
**
**************************************************************************************/

void process_directory( const char *directory_path )
{
    struct dirent *file;
    DIR *directory;

    char *file_path;

    /* this directory holds file descriptors and causes real */
    /* problems on OSX Server  , opendir causes a bus error. */

#ifdef SYSTEM_OSX
    if( strstr( directory_path, "private/dev/fd" ) )
	return;
#endif

    /* open directory, if fails, log message and continue. */

    if( ( directory = opendir( directory_path ) ) == NULL )
    {
        if( !quiet_mode )
            log_message( "==> error reading directory %s, skipping.", directory_path );

	return;
    }

    while( ( file = readdir( directory ) ) )
    {
	/* first, ignore the file if it is either the current directory */
        /* or the previous directory, this will screw up recursion.     */

  	if( strcmp( file->d_name, "." ) == 0 )
	    continue;

	if( strcmp( file->d_name, ".." ) == 0 )
	    continue;

        file_path = (char *)safemalloc( strlen( directory_path ) + 256 + 1 );

        /* create the full path of the file. */

	file_path[0] = '\0';

        strcat( file_path, directory_path );
        strcat( file_path, "/" );
        strcat( file_path, file->d_name );

	/* analyze the symlink. */

	if( file_is_symlink( file_path ) > 0 )
	    process_symlink( file_path );

	/* analyze the directory. */

	else if( file_is_directory( file_path ) > 0 )
	    process_directory( file_path );

	/* process the regular file. */
 
	else
	    process_file( file_path );

	free( file_path );
    }

    closedir( directory );
}

/**************************************************************************************
**
**    Function: process_file
**
**    Purpose:  logs info on every file if in psychonazi mode, otherwise
**		we only log executable files.
**
**    Return:   none.
**
**************************************************************************************/

void process_file( const char *file_path )
{
    int filter_result;
    entry *file_entry;

    /* run file through the filter to get authorization to log it.     */

    filter_result = file_is_allowed( &filter, file_path );

    if( filter_result > 0 )
    {
	/* if it passes through the filter, we know we can open it, so */
	/* get the entry.  If the entry was processed, we increment    */
	/* the logged counter.                                         */

	if( ( file_entry = get_entry_from_file( file_path ) ) )
	{
	    if( process_entry( file_entry ) > 0 )
	        file_logged_count++;

	    destroy_entry( file_entry );
	}
    }
    
    else if( filter_result < 0 )
    {
	if( !quiet_mode )
            log_message( "==> unaable to read file: %s, skipping.", file_path );

	file_unreadable_count++;
    }

    file_scanned_count++;

    if( !quiet_mode && ( file_scanned_count % 10 ) == 0 )
        show_current_counters();
}

/**************************************************************************************
**
**    Function: process_entry
**
**    Purpose:  logs info on every file if in psychonazi mode, otherwise
**		we only log executable files.
**
**    Return:   returns 1 if file was logged, 0 otherwise.
**
**************************************************************************************/

int process_entry( entry *file_entry )
{
    int process_result  = 0;

    if( file_entry == NULL )
	return process_result;

    /* here we simply log the entry to the database handler. */
    /* it is here that we could send the entry to another    */
    /* host on the network, if needed.                       */

    if( file_entry->key != NULL && file_entry->data != NULL )
    {
	if( database_mode )
	{
	    process_result = set_data_for_key( db, file_entry->data, file_entry->key );

	    if( !quiet_mode && process_result == 0 )
	        log_message( "==> unable to process entry for file: %s, skipping.", file_entry->key );

	    else if( !quiet_mode && process_result < 0 )
	        log_message( "==> unable to log entry for file: %s to database, skipping.", file_entry->key );
	}

	if( console_mode && ( process_result > 0 ) )
	    log_message( "%s\t%s", file_entry->data, file_entry->key );
    }

    return process_result;
}


/**************************************************************************************
**
**    Function: process_symlink
**
**    Purpose:  determine if we are traversing symlinks, and the file is a symlink to
**		a directory.  If so, process that link.
**
**    Return:   none.
**
**************************************************************************************/

void process_symlink( const char *file_path )
{
    if( symlink_mode && ( file_is_symlink_to_directory( file_path ) > 0 ) )
	process_directory( file_path );
}

/**************************************************************************************
**
**    Function: void check_arguments
**
**    Purpose:  parses command line arguments.
**
**    Return:   none.
**
**************************************************************************************/

void check_arguments( int argument_count, char *argument_list[] )
{
    int count;

    if( argument_count < 4 )
    {
        print_usage();
        exit(0);
    }

    for( count = 1; count < argument_count; count++ )
    {
        /* get directory paths. */

        if( strcmp( argument_list[count], "-i" ) == 0 )
        {
            /* skip to first path. */

	    count++;

	    while( count < argument_count  )
            {
                /* we've gone too far, move back one argument. */
		/* and break out of the loop.                  */

                if( argument_list[count][0] == '-' )
                {
		    count--;
		    break;
	  	}
                
		if( file_is_directory( argument_list[count] ) )    
		{
		    directory_list[directory_list_size] = safemalloc( strlen( argument_list[count] ) );
		    strcpy( directory_list[directory_list_size++], argument_list[count++] );
		}

		else
		{
		    if( !quiet_mode )
		        log_message( "==> directory: %s does not exist.", argument_list[count] );

		    exit(0);
		}
	    }
        }

        else if( strcmp( argument_list[count], "-h" ) == 0 )
        {
            print_usage();
            exit(0);
	}

        else if( strcmp( argument_list[count], "-v" ) == 0 )
	    console_mode = 1;

        else if( strcmp( argument_list[count], "-x" ) == 0 )
	    set_file_filter( &filter, FILTER_EXECUTABLE );

	else if( strcmp( argument_list[count], "-l" ) == 0 )
	    symlink_mode = 1;

	else if( strcmp( argument_list[count], "-q" ) == 0 )
	    quiet_mode = 1;

	else if( strcmp( argument_list[count], "-O" ) == 0 )
	    force_mode = 1;

        else if( strcmp( argument_list[count], "-psychonazi" ) == 0 )
	    set_file_filter( &filter, FILTER_ALL );

 	else if( strcmp( argument_list[count], "-o" ) == 0 && ( count < ( argument_count - 1 ) ) )
	{
            strcpy( database_path, argument_list[++count] );
	    database_mode = 1;
	}

	else
        {
	    print_usage();
	    exit(0);
	}
    }

    /* user must specify a database to write to. */

    if( !database_mode )
    {

	if( quiet_mode )
	    exit(0);

	print_usage();
	log_message_exit( "error: a database must be specified.\n" );
    }
}


/**************************************************************************************
**
**    Function: void open_database_files
**
**    Purpose:  tries to open the database files, if they exists, prompts
**		for overwrite.
**
**    Return:   none.
**
**************************************************************************************/

void open_database_files()
{
    if( !database_mode )
	return;

    /* database exists. */

    if( access( database_path, F_OK ) == 0 )
    {
	if( !force_mode )
        { 
            printf( "\ndatabase: %s already exists, overwrite? [y/n] ", database_path );

	    if( getc( stdin ) == 'n' )
	        log_message_exit( "...exiting" );
	}

	if( unlink( database_path ) != 0 )
        {
	    if( force_mode )
	        exit(0);

	    else
	        log_message_exit( "unable to overwrite old database, exiting." );
	}
    }

    if( ( db = create_database_for_write( database_path ) ) == NULL )
    {
	if( !quiet_mode )
            log_message_exit( "unable to open database %s", database_path );
    }
}

/**************************************************************************************
**
**    Function: void close_database_files
**
**    Purpose:  tries to close the database files.
**
**    Return:   none.
**
**************************************************************************************/

void close_database_files()
{
    if( !database_mode )
	return;

    close_database( db );
}


/**************************************************************************************
**
**    Function: void print_usage
**
**    Purpose:  prints usage statement and command line arguments for
**		using osiris.
**
**    Return:   none.
**
**************************************************************************************/

void print_usage()
{
    log_message( "" );
    log_message( "usage: osiris -i <path>... -o <path> [-v] [-x] [-l] [-q] [-h] [-O] [-psychonazi]" );
    log_message( "" );

    log_message( "     -i               directories to search in ( recursive )." );
    log_message( "     -o               path to export the datafile to. ( e.g., /var/log/osiris )." );
    log_message( "     -v               dump output to console, default if -o is not used." );
    log_message( "     -h               display these messages." );
    log_message( "     -q               quiet mode, not output to console." );
    log_message( "     -x               log -only- those files with the executable bit set." );
    log_message( "     -l               traverse symlinks to other directories." );
    log_message( "     -O               no prompt for database overwrite ( use with -q guarantees no console output )." );
    log_message( "     -psychonazi      log every file in the directory specified, as permitted." );

    print_program_info();
}


/**************************************************************************************
**
**    Function: void print_program_info
**
**    Purpose:  prints program name, version, authors, etc.
**
**    Return:   none.
**
**************************************************************************************/

void print_program_info()
{
    log_message( "" );

    log_message( "-------------------------------------------" );
    log_message( "  %s  - version %s                         ", program_name, program_version );
    log_message( "  www.shmoo.com                 (c) 2000   " );
    log_message( "-------------------------------------------" );

    log_message( "" );
}


/**************************************************************************************
**
**    Function: void print_startup_info
**
**    Purpose:  prints program startup information so that
**              the user isn't left hanging for output.
**
**    Return:   none.
**
**************************************************************************************/

void print_startup_info()
{
    int index;

    print_program_info();
    printf( "scanning directories: " );

    for( index = 0; index < directory_list_size; index++ )
	printf( "%s ", directory_list[index] );

    printf( "\n" );
    fflush( stdout );
}

/**************************************************************************************
**
**    Function: show_current_counters
**
**    Purpose:  display the current total files surveyed, and files logged
**		to standard output so that the user knows things are working.
**
**    Return:   none.
**
**************************************************************************************/

void show_current_counters()
{
    int index;

    printf( "*=> files scanned: %d  files logged: %d", file_scanned_count, file_logged_count );

    /* remove old counter line. */

    for( index = 0; index < 50; index++ )
        printf( "%c", 0x8 );
}


/**************************************************************************************
**
**    Function: quit
**
**    Purpose:  go through all of the directories specified by the user
**		in the directory_list and free them, close the database
**		files.
**
**    Return:   none.
**
**************************************************************************************/

void quit()
{
    int index;

    time_t current_time = time( NULL );
    strcpy( end_time, ctime( &current_time ) );
    start_time[strlen( end_time ) - 1] = '\0';

    /* free directory list memory. */

    for( index = 0; index < directory_list_size; index++ )
	free( directory_list[index] );

    close_database_files();

    if( !quiet_mode )
        print_results();
}

/**************************************************************************************
**
**    Function: print_results
**
**    Purpose:  print the counters to standard output and print the time stamps.
**
**    Return:   none.
**
**************************************************************************************/

void print_results()
{
    log_message( "\n" );

    log_message( "files logged:       %d", file_logged_count );
    log_message( "files scanned:      %d", file_scanned_count );
    log_message( "files not readable: %d", file_unreadable_count );

    log_message( "" );

    log_message( "[ start time ] : %s", start_time );
    log_message( "[ end time   ] : %s", end_time );
}
